package info.buscalivro.buscalivro;

import android.app.ProgressDialog;
import android.support.design.widget.Snackbar;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.support.v7.widget.SearchView;
import android.support.v7.widget.Toolbar;
import android.view.Gravity;
import android.view.View;
import android.widget.AdapterView;
import android.widget.CompoundButton;
import android.widget.EditText;
import android.widget.TextView;

import com.mikepenz.materialdrawer.Drawer;
import com.mikepenz.materialdrawer.model.PrimaryDrawerItem;
import com.mikepenz.materialdrawer.model.SectionDrawerItem;
import com.mikepenz.materialdrawer.model.SwitchDrawerItem;
import com.mikepenz.materialdrawer.model.interfaces.IDrawerItem;
import com.mikepenz.materialdrawer.model.interfaces.OnCheckedChangeListener;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import info.buscalivro.buscalivro.fragments.FrequentlyQuestionsFragment;
import info.buscalivro.buscalivro.fragments.HelpTopicsFragment;
import info.buscalivro.buscalivro.fragments.ReportErrorFragment;
import info.buscalivro.buscalivro.interfaces.DealResponse;
import info.buscalivro.buscalivro.objects.BookElement;
import info.buscalivro.buscalivro.objects.SearchObj;
import info.buscalivro.buscalivro.fragments.HomeFragment;
import info.buscalivro.buscalivro.fragments.ResultsFragment;


public class MainActivity extends AppCompatActivity {
    private Toolbar mToolbar;
    private SearchView mSearchView;
    private Drawer.Result mNavigationDrawer;
    private FragmentManager fm; /* FragmentManager to make transactions with fragments */
    private SearchObj search = new SearchObj(""); /* object of search params */
    private List<BookElement> mList = new ArrayList<>(); /* list of objects in ResultsFragment */
    private List<String> mFieldList = new ArrayList<>(); /* list of fields checked in navigation */
    private boolean notReload = true;
    public String csrfToken;
    private ProgressDialog dialog;

    /**
     * Create Activity, Toolbar, Navigation Drawer and HomeFragment
     */
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        createToolbar(); /* create Toolbar */
        createNavigationDrawer(savedInstanceState); /* create Navigation Drawer */
        addNavigationDrawerItems(); /* add items in Navigation Drawer */

        if (savedInstanceState == null) { /* if first instance, create and add HomeFragment in stack */
            FragmentTransaction ft; /* this is declared every use because global declaration make bugs */
            fm = getSupportFragmentManager();
            ft = fm.beginTransaction();
            HomeFragment homeFragment = new HomeFragment();
            ft.setTransition(FragmentTransaction.TRANSIT_FRAGMENT_OPEN);
            ft.add(R.id.container, homeFragment, "HomeFragment"); /* Show HomeFragment, don't put in stack */
            ft.commit();

            dialog = ProgressDialog.show(this, null, "Testando comunicação, aguarde...", true);
            Utils.getTokenRequest(this, new DealResponse() {
                @Override
                public void withResponse(String response) {
                    SetCsrftoken(response);
                    dialog.dismiss();
                }

                @Override
                public void withoutResponse() {
                    dialog.dismiss();
                    Utils.showDialog(MainActivity.this, "Erro de comunicação", "Desculpe, algo de errado aconteceu. Verifique sua conexão", true);
                }
            });
        }
    }

    /**
     * Create Toolbar and SearchView linked
     */
    private void createToolbar() {
        /* set Toolbar */
        mToolbar = (Toolbar) findViewById(R.id.tb_main);
        setSupportActionBar(mToolbar);

        /* create SearchView */
        mSearchView = new SearchView(this);
        setSearchViewFocusable(false); /* make SearchView unfocused */
        mSearchView.setOnQueryTextListener(new SearchFilter());
        mSearchView.setLayoutParams(new Toolbar.LayoutParams(Gravity.END)); /* set SearchView on right */
        mToolbar.addView(mSearchView); /* add SearchView in Toolbar */
    }

    /**
     * Set the SearchView is shown or not
     */
    public void setSearchViewFocusable(boolean choice) {
        mSearchView.setFocusable(choice);
    }

    /**
     * Class to implement SearchView
     */
    class SearchFilter implements SearchView.OnQueryTextListener {
        @Override
        public boolean onQueryTextSubmit(String query) {
            if (checkResultsFragment()) { /* if actual fragment is ResultsFragment */
                search.setQueryText(query); /* set text in SearchView to search object */
                Utils.closeKeyboard(MainActivity.this);
                clearStackFragments();
                callResultsFragment(); /* recall ResultsFragment */
            }
            return true; /* tells the SearchView that the app dealt with the request  */
        }

        @Override
        public boolean onQueryTextChange(String newText) {
            return false;
        }
    }

    /**
    * On Back Button is pressed */
    @Override
    public void onBackPressed() {
        /* if navigation drawer was open, on back button pressed, close navigation */
        if (mNavigationDrawer.isDrawerOpen()) {
            mNavigationDrawer.closeDrawer();
        } else {
            super.onBackPressed();
        }
    }

    /**
     * Create the Navigation Drawer without elements
     */
    private void createNavigationDrawer(Bundle savedStanceState) {
        mNavigationDrawer = new Drawer()
                .withActivity(this) /* set Activity */
                .withToolbar(mToolbar) /* set Toolbar */
                .withDisplayBelowToolbar(true) /* the Navigation does not overrides Toolbar */
                .withActionBarDrawerToggle(true) /* set Toolbar button to Navigation */
                .withSavedInstance(savedStanceState) /* set savedStanceState */
                .withActionBarDrawerToggleAnimated(true) /* set animation to button in Toolbar */
                .withSelectedItem(1) /* set first item (0 is section) selected */
                .withOnDrawerListener(new Drawer.OnDrawerListener() { /* implements listener to actions with Navigation */
                    @Override
                    public void onDrawerOpened(View view) {
                        Utils.closeKeyboard(MainActivity.this);
                    }

                    @Override
                    public void onDrawerClosed(View view) {
                        /* if all "Campos" switch are disable */
                        if (mFieldList.isEmpty()) {
                            Snackbar.make(findViewById(android.R.id.content), getString(R.string.SelectFieldMsg), Snackbar.LENGTH_LONG).show();
                            mNavigationDrawer.openDrawer();
                        } else if (search.getKinds().isEmpty()) { /* if all "Tipos" switch are disable */
                            Snackbar.make(findViewById(android.R.id.content), getString(R.string.SelectFieldMsg), Snackbar.LENGTH_LONG).show();
                            mNavigationDrawer.openDrawer();
                        } else if (checkResultsFragment()) { /* if actual fragment are ResultsFragment */
                            if (!notReload) {
                                callResultsFragment();
                            }
                        }
                    }

                    @Override
                    public void onDrawerSlide(View view, float v) {
                    }
                })
                .withOnDrawerItemClickListener(new Drawer.OnDrawerItemClickListener() {
                    @Override
                    public void onItemClick(AdapterView<?> adapterView, View view, int i, long l, IDrawerItem iDrawerItem) {
                        FragmentTransaction ft;
                        TextView libraryToSearch = (TextView) findViewById(R.id.libraryToSearch);
                        switch (i) { /* switch on items in Navigation */
                            case 1: /* Todas */
                                search.setLibrary("");
                                clearStackFragments();
                                if (checkResultsFragment()) {
                                    mList.clear();
                                    notReload = true;
                                    callResultsFragment();
                                } else try {
                                        libraryToSearch.setText(getString(R.string.searchingAllLibrariesTextView));
                                    } catch (NullPointerException ignored) {}
                                break;
                            case 2: /* Biblioteca da UEMS */
                                search.setLibrary("1");
                                clearStackFragments();
                                if (checkResultsFragment()) {
                                    mList.clear();
                                    notReload = true;
                                    callResultsFragment();
                                } else try {
                                        libraryToSearch.setText(getString(R.string.searchingUEMSTextView));
                                    } catch (NullPointerException ignored) {}
                                break;
                            case 3: /* Biblioteca da UFGD */
                                search.setLibrary("2");
                                clearStackFragments();
                                if (checkResultsFragment()) {
                                    mList.clear();
                                    notReload = true;
                                    callResultsFragment();
                                } else try {
                                        libraryToSearch.setText(getString(R.string.searchingUFGDTextView));
                                    } catch (NullPointerException ignored) {}
                                break;
                            case 14: /* Perguntas Frequentes */
                                ft = fm.beginTransaction();
                                FrequentlyQuestionsFragment frequentlyQuestionsFragment = new FrequentlyQuestionsFragment();
                                ft.setTransition(FragmentTransaction.TRANSIT_FRAGMENT_OPEN);
                                ft.replace(R.id.container, frequentlyQuestionsFragment, "FrequentlyQuestionsFragment");
                                ft.addToBackStack("stack");
                                ft.commit();
                                break;
                            case 15: /* Tópicos de Ajuda */
                                ft = fm.beginTransaction();
                                HelpTopicsFragment helpTopicsFragment = new HelpTopicsFragment();
                                ft.setTransition(FragmentTransaction.TRANSIT_FRAGMENT_OPEN);
                                ft.replace(R.id.container, helpTopicsFragment, "HelpTopicsFragment");
                                ft.addToBackStack("stack");
                                ft.commit();
                                break;
                            case 16: /* Reportar um Erro */
                                ft = fm.beginTransaction();
                                ReportErrorFragment reportErrorFragment = new ReportErrorFragment();
                                ft.setTransition(FragmentTransaction.TRANSIT_FRAGMENT_OPEN);
                                ft.replace(R.id.container, reportErrorFragment, "ReportErrorFragment");
                                ft.addToBackStack("stack");
                                ft.commit();
                                break;
                            case 17: /* Avaliar o Aplicativo */
                                break;
                            default:
                                break;
                        }
                    }
                })
                .build();
        mNavigationDrawer.keyboardSupportEnabled(this, true);
    }

    /**
     * Listener to Field Switch in Navigation
     */
    private OnCheckedChangeListener mFieldListener = new OnCheckedChangeListener() {
        @Override
        public void onCheckedChanged(IDrawerItem iDrawerItem, CompoundButton compoundButton, boolean b) {
            SwitchDrawerItem item = (SwitchDrawerItem) iDrawerItem;
            String itemName = item.getName();
            if (b) { /* if checked */
                if (!mFieldList.contains(itemName)) {
                    mFieldList.add(itemName);
                }
            } else { /* if unchecked */
                if (mFieldList.contains(itemName)) {
                    mFieldList.remove(itemName);
                }
            }

            if (mFieldList.size() == 2) { /* if all switch fields are checked */
                search.setField(""); /* all fields */
            } else {
                if (!mFieldList.isEmpty()) {
                    if (mFieldList.get(0).equals("Título")) {
                        search.setField("title");
                    } else if (mFieldList.get(0).equals("Autor")) {
                        search.setField("author");
                    }
                } else {
                    search.setField("");
                }
            }
            notReload = false;
        }
    };

    /**
     * Listener to Types Switch in Navigation
     */
    private OnCheckedChangeListener mTypeListener = new OnCheckedChangeListener() {
        @Override
        public void onCheckedChanged(IDrawerItem iDrawerItem, CompoundButton compoundButton, boolean b) {
            SwitchDrawerItem item = (SwitchDrawerItem) iDrawerItem;
            String itemName = item.getName();
            if (b) { /* if checked */
                if (!search.containKind(itemName)) {
                    search.addKind(itemName);
                }
            } else { /* if unchecked */
                if (search.containKind(itemName)) {
                    search.removeKind(itemName);
                }
            }
            notReload = false;
        }
    };

    /**
     * Create items in Navigation Drawer
     */
    private void addNavigationDrawerItems() {
        mNavigationDrawer.addItem(new SectionDrawerItem().withName(getString(R.string.LibrariesNavigationSection)));
        mNavigationDrawer.addItem(new PrimaryDrawerItem().withName(getString(R.string.AllLibrariesNavigation)).withIcon(R.drawable.ic_all_library));
        mNavigationDrawer.addItem(new PrimaryDrawerItem().withName(getString(R.string.UEMSLibraryNavigation)).withIcon(R.drawable.ic_uems));
        mNavigationDrawer.addItem(new PrimaryDrawerItem().withName(getString(R.string.UFGDLibraryNavigation)).withIcon(R.drawable.ic_ufgd));
        mNavigationDrawer.addItem(new SectionDrawerItem().withName(getString(R.string.FieldsSectionNavigation)));
        mNavigationDrawer.addItem(new SwitchDrawerItem().withName(getString(R.string.TitleSwitchNavigation)).withChecked(true).withOnCheckedChangeListener(mFieldListener));
        mNavigationDrawer.addItem(new SwitchDrawerItem().withName(getString(R.string.AuthorSwitchNavigation)).withChecked(true).withOnCheckedChangeListener(mFieldListener));
        mNavigationDrawer.addItem(new SectionDrawerItem().withName(getString(R.string.KindsSectionNavigation)));
        mNavigationDrawer.addItem(new SwitchDrawerItem().withName(getString(R.string.BooksSwitchKindNavigation)).withChecked(true).withOnCheckedChangeListener(mTypeListener));
        mNavigationDrawer.addItem(new SwitchDrawerItem().withName(getString(R.string.NewsPapersSwitchKindNavigation)).withChecked(true).withOnCheckedChangeListener(mTypeListener));
        mNavigationDrawer.addItem(new SwitchDrawerItem().withName(getString(R.string.MonographTCCSwitchKindNavigation)).withChecked(true).withOnCheckedChangeListener(mTypeListener));
        mNavigationDrawer.addItem(new SwitchDrawerItem().withName(getString(R.string.ThesisDissertationSwitchKindNavigation)).withChecked(true).withOnCheckedChangeListener(mTypeListener));
        mNavigationDrawer.addItem(new SwitchDrawerItem().withName(getString(R.string.OtherSwitchKindsNavigation)).withChecked(true).withOnCheckedChangeListener(mTypeListener));
        mNavigationDrawer.addItem(new SectionDrawerItem().withName(getString(R.string.HelpSectionNavigation)));
        mNavigationDrawer.addItem(new PrimaryDrawerItem().withName(getString(R.string.FrequentlyQuestionsNavigation)).withIcon(R.drawable.ic_help));
        mNavigationDrawer.addItem(new PrimaryDrawerItem().withName(getString(R.string.HelpTopicsNavigation)).withIcon(R.drawable.ic_topics));
        mNavigationDrawer.addItem(new PrimaryDrawerItem().withName(getString(R.string.ReportErrorNavigation)).withIcon(R.drawable.ic_report));
        mNavigationDrawer.addItem(new PrimaryDrawerItem().withName(getString(R.string.EvaluateAppNavigation)).withIcon(R.drawable.ic_avaliable));

        mFieldList.addAll( /* all fields checked */
                Arrays.asList(
                        getString(R.string.TitleSwitchNavigation),
                        getString(R.string.AuthorSwitchNavigation)));

        ArrayList<String> typesList = new ArrayList<>();
        typesList.addAll( /* all types checked */
                Arrays.asList(
                        getString(R.string.BooksSwitchKindNavigation),
                        getString(R.string.NewsPapersSwitchKindNavigation),
                        getString(R.string.MonographTCCSwitchKindNavigation),
                        getString(R.string.ThesisDissertationSwitchKindNavigation),
                        getString(R.string.OtherSwitchKindsNavigation)));
        search.setKinds(typesList);
    }

    /**
     *  Clear stack of fragments
     *  HomeFragment are not stacked, because this, the clearStackFragments
     *  function remove all stacked fragments, leaving only HomeFragment on screen
     */
    public void clearStackFragments() {
        for(int i = 0; i < fm.getBackStackEntryCount(); i++) {
            fm.popBackStack(null, FragmentManager.POP_BACK_STACK_INCLUSIVE);
        }
    }

    /**
     * Set title, subtitle and if SearchView is visible or not
     */
    public void setToolbar(String title, String subtitle, boolean searchVisible) {
        mToolbar.setTitle(title); /* set Toolbar title */
        mToolbar.setSubtitle(subtitle); /* set Toolbar subtitle */

        if (searchVisible) { /* show SearchView */
            mSearchView.setQuery(search.getQueryText(), false); /* set the input query in SearchView */
            mSearchView.setVisibility(View.VISIBLE);
        } else { /* hide SearchView */
            mSearchView.setVisibility(View.GONE);
        }
    }

    /**
     * Listener of button "#Pesquisar"
     * Get params and make search
     */
    public void makeSearch(View view) {
        EditText searchField = (EditText) findViewById(R.id.search_field);

        if (!searchField.getText().toString().isEmpty()) {
            search.setQueryText(searchField.getText().toString()); /* set text of user */
            search.setField("");

            callResultsFragment();
        } else {
            Snackbar.make(findViewById(android.R.id.content), getString(R.string.FillSearchMsg), Snackbar.LENGTH_LONG).show();
        }
    }

    /**
     * Create, add to stack and show ResultsFragment
     */
    public void callResultsFragment() {
        /* create and add ResultFragment in stack */
        Utils.closeKeyboard(this);
        FragmentTransaction ft;
        ft = fm.beginTransaction();
        ResultsFragment resultsFragment = new ResultsFragment();
        ft.setTransition(FragmentTransaction.TRANSIT_FRAGMENT_OPEN);
        ft.replace(R.id.container, resultsFragment, "ResultsFragment");
        ft.addToBackStack("stack");
        ft.commit();
    }

    /**
     * Check if visible fragment is ResultsFragment
     */
    private boolean checkResultsFragment() {
        ResultsFragment listFragment = (ResultsFragment) getSupportFragmentManager().findFragmentByTag("ResultsFragment");
        return (listFragment != null && listFragment.isVisible());
    }

    /**
     * Get SearchObj
     */
    public SearchObj getSearch() {
        return search;
    }

    /**
     * Get result items list
     */
    public List<BookElement> getmList() {
        return mList;
    }

    /**
     * Get element of result items list
     */
    public BookElement getElementList(int position) {
        return mList.get(position);
    }

    public String getCsrftoken() {
        return this.csrfToken;
    }

    public void SetCsrftoken(String csrfToken) {
        this.csrfToken = csrfToken;
    }
}
