package info.buscalivro.buscalivro;

import android.app.Activity;
import android.content.Context;
import android.content.DialogInterface;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.support.v7.app.AlertDialog;
import android.text.TextUtils;

import com.android.volley.AuthFailureError;
import com.android.volley.Request;
import com.android.volley.RequestQueue;
import com.android.volley.Response;
import com.android.volley.VolleyError;
import com.android.volley.toolbox.StringRequest;
import com.android.volley.toolbox.Volley;
import com.mikepenz.materialdrawer.util.KeyboardUtil;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import info.buscalivro.buscalivro.interfaces.DealResponse;
import info.buscalivro.buscalivro.objects.BookElement;
import info.buscalivro.buscalivro.objects.SearchObj;

public class Utils {
    public static int hits;
    public static String domain = "http://buscalivro.info/";

    public static int getNumRows() {
        return 20;
    }

    private static String getDataUrl() {
        return domain + "mobile_search_server/";
    }

    protected static String getCsrfTokenUrl() {
        return domain + "get_cookie/";
    }

    protected static String getAvaliablesUrl() {
        return domain + "get_avaliables/";
    }

    private static String getErrorUrl() {
        return domain + "mobile_report_error/";
    }

    public static void showDialog(final Context context, final String title, final String message, final boolean returnFragment) {
        AlertDialog.Builder builder = new AlertDialog.Builder(context);
        builder.setPositiveButton(context.getString(R.string.CloseButtonDialog), new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int id) {
                if(returnFragment) {
                    ((MainActivity) context).clearStackFragments();
                }
            }
        })
                .setTitle(title)
                .setMessage(message)
                .show();
    }

    public static void callDataResults(final Context context, final SearchObj search, final int offset, final DealResponse inter) {
        StringRequest request = new StringRequest(Request.Method.POST, getDataUrl(), new Response.Listener<String>() {
            @Override
            public void onResponse(String response) {
                inter.withResponse(response);
            }
        }, new Response.ErrorListener() {
            @Override
            public void onErrorResponse(VolleyError error) {
                inter.withoutResponse();
                Utils.showDialog(context, context.getString(R.string.TitleErrorConnectionDialog), context.getString(R.string.MsgErrorConnectionDialog) + " " + search.getField(), true);
            }
        }) {
            @Override
            public Map<String, String> getParams() throws AuthFailureError {
                Map<String, String> params = new HashMap<>();
                params.put("query", search.getQueryText());
                params.put("library", search.getLibrary());
                params.put("kind", TextUtils.join(", ", search.getKinds()));
                params.put("field", search.getField());
                params.put("fl", search.getFl());
                params.put("start", ""+offset);
                params.put("rows", ""+ Utils.getNumRows());
                return(params);
            }
            @Override
            public Map<String, String> getHeaders() throws AuthFailureError {
                Map<String, String> headers = new HashMap<>();
                headers.put("X-CSRFToken", ((MainActivity) context).getCsrftoken());
                headers.put("Cookie", "csrftoken=" + ((MainActivity) context).getCsrftoken());
                return headers;
            }
        };
        request.setTag("callDataResults");
        RequestQueue rq = Volley.newRequestQueue(context);
        rq.add(request);
    }

    public static List<BookElement> addItems(final String response) {
        // convert response to object
        List<BookElement> auxList = new ArrayList<>();
        JSONObject jo;

        try {
            JSONArray ja = new JSONArray(response);
            for(int i = 0; i < ja.length() - 1; i++) {
                BookElement book = new BookElement();
                jo = ja.getJSONObject(i);

                if(jo.has("title")) { book.setTitle(jo.getString("title")); } else { book.setTitle(""); }
                if(jo.has("author")) { book.setAuthor(jo.getString("author")); } else { book.setAuthor(""); }
                if(jo.has("library")) { book.setLibrary(jo.getString("library")); } else { book.setLibrary(""); }
                if(jo.has("edition")) { book.setEdition(jo.getString("edition")); } else { book.setEdition(""); }
                if(jo.has("publication")) { book.setPublication(jo.getString("publication")); } else { book.setPublication(""); }
                if(jo.has("cdd")) { book.setCdd(jo.getString("cdd")); } else { book.setCdd(""); }
                if(jo.has("pha")) { book.setPha(jo.getString("pha")); } else { book.setPha(""); }
                if(jo.has("kind")) { book.setKind(jo.getString("kind")); } else { book.setKind(""); }
                if(jo.has("description")) { book.setDescription(jo.getString("description")); } else { book.setDescription(""); }
                if(jo.has("field")) { book.setField(jo.getString("field")); } else { book.setField(""); }
                auxList.add(book);
            }
            jo = ja.getJSONObject(ja.length() - 1);
            if(jo.has("hits")) { Utils.hits = Integer.parseInt(jo.getString("hits")); } else { Utils.hits = 0; }
        } catch (JSONException e) {
            e.printStackTrace();
        }
        return auxList;
    }

    public static void sendError(final Context context, final String report_text, final DealResponse inter) {
        StringRequest request = new StringRequest(Request.Method.POST, getErrorUrl(), new Response.Listener<String>() {
            @Override
            public void onResponse(String response) {
                inter.withResponse(response);
            }
        }, new Response.ErrorListener() {
            @Override
            public void onErrorResponse(VolleyError error) {
                inter.withoutResponse();
            }
        }) {
            @Override
            public Map<String, String> getParams() throws AuthFailureError {
                Map<String, String> params = new HashMap<>();
                params.put("reportError", report_text);
                return(params);
            }
            @Override
            public Map<String, String> getHeaders() throws AuthFailureError {
                Map<String, String> headers = new HashMap<>();
                headers.put("X-CSRFToken", ((MainActivity) context).getCsrftoken());
                headers.put("Cookie", "csrftoken=" + ((MainActivity) context).getCsrftoken());
                return headers;
            }
        };
        request.setTag("sendError");
        RequestQueue rq = Volley.newRequestQueue(context);
        rq.add(request);
    }

    public static boolean isConnected(final Context context) {
        ConnectivityManager cm = (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo netInfo = cm.getActiveNetworkInfo();
        return (netInfo != null && netInfo.isConnected());
    }

    public static void closeKeyboard(final Context context) {
        KeyboardUtil.hideKeyboard((Activity) context);
    }

    public static void getTokenRequest(final Context context, final DealResponse inter) {
        StringRequest request = new StringRequest(Request.Method.GET, Utils.getCsrfTokenUrl(), new Response.Listener<String>() {
            @Override
            public void onResponse(String response) {
                inter.withResponse(response);
            }
        }, new Response.ErrorListener() {
            @Override
            public void onErrorResponse(VolleyError error) {
                inter.withoutResponse();
            }
        }) {
            @Override
            public Map<String, String> getParams() throws AuthFailureError {
                Map<String, String> params = new HashMap<>();
                params.put("from", "Android");
                return (params);
            }
        };
        request.setTag("callDataResults");
        RequestQueue rq = Volley.newRequestQueue(context);
        rq.add(request);
    }

    public static void getAvaliables(final Context context, final String library, final String field, final DealResponse inter) {
        StringRequest request = new StringRequest(Request.Method.POST, Utils.getAvaliablesUrl(), new Response.Listener<String>() {
            @Override
            public void onResponse(String response) {
                inter.withResponse(response);
            }
        }, new Response.ErrorListener() {
            @Override
            public void onErrorResponse(VolleyError error) {
                inter.withoutResponse();
            }
        }) {
            @Override
            public Map<String, String> getParams() throws AuthFailureError {
                Map<String, String> params = new HashMap<>();
                params.put("library", library);
                params.put("field", field);
                return (params);
            }
            @Override
            public Map<String, String> getHeaders() throws AuthFailureError {
                Map<String, String> headers = new HashMap<>();
                headers.put("X-CSRFToken", ((MainActivity) context).getCsrftoken());
                headers.put("Cookie", "csrftoken=" + ((MainActivity) context).getCsrftoken());
                return headers;
            }
        };
        request.setTag("callDataResults");
        RequestQueue rq = Volley.newRequestQueue(context);
        rq.add(request);
    }
}
