package info.buscalivro.buscalivro.fragments;

import android.app.ProgressDialog;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Toast;

import java.util.List;

import info.buscalivro.buscalivro.R;
import info.buscalivro.buscalivro.interfaces.DealResponse;
import info.buscalivro.buscalivro.MainActivity;
import info.buscalivro.buscalivro.interfaces.RecycleViewOnClickListenerHack;
import info.buscalivro.buscalivro.objects.BookAdapter;
import info.buscalivro.buscalivro.objects.BookElement;
import info.buscalivro.buscalivro.objects.SearchObj;
import info.buscalivro.buscalivro.Utils;


public class ResultsFragment extends Fragment implements RecycleViewOnClickListenerHack {
    private RecyclerView mRecyclerView;
    private ProgressDialog dialog;
    private int offset = 0;
    private SearchObj search;
    private boolean onBottom = false;
    private boolean firstTime = false;
    private int hits = 0;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        ((MainActivity) getActivity()).getmList().clear();

        firstTime = true;
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View view  = inflater.inflate(R.layout.result_fragment, container, false);

        mRecyclerView = (RecyclerView) view.findViewById(R.id.recycleview_list);
        createRecycleView();

        search = ((MainActivity) getActivity()).getSearch();

        if (firstTime) {
            dialog = ProgressDialog.show(getActivity(), null, getActivity().getString(R.string.LoadingDataMsgDialog), true);
            Utils.callDataResults(getActivity(), search, offset, new DealResponse() {
                @Override
                public void withResponse(String response) {
                    makeList(response);
                }

                @Override
                public void withoutResponse() {
                    dialog.dismiss();
                }
            });
            firstTime = false;
        }

        return view;
    }

    @Override
    public void onResume() {
        super.onResume();

        ((MainActivity) getActivity()).setToolbar(getActivity().getString(R.string.ResultsToolbarTitle), Utils.hits + getActivity().getString(R.string.ResultsFoundSubtitleToolbar), true);
        ((MainActivity) getActivity()).setSearchViewFocusable(false);
    }

    @Override
    public void onClickListener(View view, int position) {
        FragmentTransaction ft;
        FragmentManager fm = getActivity().getSupportFragmentManager();
        Bundle extra = new Bundle();
        DetailsFragment detailsFragment = new DetailsFragment();
        ft = fm.beginTransaction();

        extra.putInt("position", position);
        detailsFragment.setArguments(extra);
        ft.setTransition(FragmentTransaction.TRANSIT_FRAGMENT_OPEN);
        ft.replace(R.id.container, detailsFragment, "DetailsFragment");
        ft.addToBackStack("stack");
        ft.commit();
    }

    private void createRecycleView() {
        mRecyclerView.setHasFixedSize(true);
        mRecyclerView.setOnScrollListener(new RecyclerView.OnScrollListener() {
            @Override
            public void onScrollStateChanged(RecyclerView recyclerView, int newState) {
                super.onScrollStateChanged(recyclerView, newState);
            }

            @Override
            public void onScrolled(RecyclerView recyclerView, int dx, int dy) {
                super.onScrolled(recyclerView, dx, dy);

                LinearLayoutManager llm = (LinearLayoutManager) mRecyclerView.getLayoutManager();

                if (((MainActivity) getActivity()).getmList().size() == llm.findLastCompletelyVisibleItemPosition() + 1) {
                    if (!onBottom) {
                        dialog = ProgressDialog.show(getActivity(), null, getActivity().getString(R.string.LoadingMoreDataDialog), true);
                        Utils.callDataResults(getActivity(), search, offset, new DealResponse() {
                            @Override
                            public void withResponse(String response) {
                                makeList(response);
                            }

                            @Override
                            public void withoutResponse() {
                                dialog.dismiss();
                            }
                        });
                    } else {
                        Toast.makeText(getActivity(), getActivity().getString(R.string.NoMoreDataDialogTitleToolbar), Toast.LENGTH_SHORT).show();
                    }
                }
            }
        });

        LinearLayoutManager llm = new LinearLayoutManager(getActivity());
        llm.setOrientation(LinearLayoutManager.VERTICAL);
        mRecyclerView.setLayoutManager(llm);

        BookAdapter adapter = new BookAdapter(getActivity(), ((MainActivity) getActivity()).getmList());
        adapter.setRecycleViewOnClickListenerHack(ResultsFragment.this);
        mRecyclerView.setAdapter(adapter);
    }

    private void makeList(String response) {
        if(!response.equals("[]")) {
            List<BookElement> auxList;
            auxList = Utils.addItems(response);

            if (((MainActivity) getActivity()).getmList().isEmpty()) {
                BookAdapter adapter = new BookAdapter(getActivity(), ((MainActivity) getActivity()).getmList());
                adapter.setRecycleViewOnClickListenerHack(ResultsFragment.this);
                mRecyclerView.setAdapter(adapter);
            }
            BookAdapter adapter = (BookAdapter) mRecyclerView.getAdapter();

            for (int i = 0; i < auxList.size(); i++) {
                adapter.addListItem(auxList.get(i), ((MainActivity) getActivity()).getmList().size());
            }

            offset += Utils.getNumRows();
            ((MainActivity) getActivity()).setToolbar(getActivity().getString(R.string.ResultsToolbarTitle), Utils.hits + getActivity().getString(R.string.ResultsFoundSubtitleToolbar), true);
            dialog.dismiss();
        } else {
            dialog.dismiss();
            if(((MainActivity) getActivity()).getmList().size() > 0) {
                Toast.makeText(getActivity(), getActivity().getString(R.string.NoMoreDataDialogTitleToolbar), Toast.LENGTH_SHORT).show();
                onBottom = true;
            } else {
                Utils.showDialog(getActivity(), getActivity().getString(R.string.NothingFoundDialogTitle), getActivity().getString(R.string.NothingFoundDialogSubtitle), true);
            }
        }
    }

    public int getHits() {
        return hits;
    }

    public void setHits(int hits) {
        this.hits = hits;
    }
}
