# -*- coding: utf 8 -*-
import json
from datetime import date

from django.conf import settings
from django.http import HttpResponse, Http404
from django.shortcuts import render
from django.views.decorators.csrf import ensure_csrf_cookie

from search_server.utils import query_solr, search_Q
from web_scraping.utils import book_avaliable_uems, book_avaliable_ufgd

from search_server.models import Library


@ensure_csrf_cookie
def index(request):
    context = {}
    context['year'] = date.today().year
    context['url'] = settings.DOMAIN

    try:
        context['q'] = request.GET['q']
        context['field'] = request.GET['field']
        context['librarys'] = Library.objects.all()
        return render(request, 'search.html', context)
    except:
        return render(request, 'index.html', context)


def get_books_grid(request):
    try:
        query = json.dumps(request.POST)
        query = json.loads(query)
        if query['kind']:
            query['kind'] = query['kind'].split(', ')
        search, qf = search_Q(query)
        response = query_solr(q=search, qf=qf, rows=query['limit'], start=query['offset'], highlight=True)
        data = {}
        data['hits'] = response.hits
        data['books'] = response.docs
        data['highlights'] = response.highlighting
        return HttpResponse(json.dumps(data), content_type='application/json')
    except:
        raise Http404


def get_book_info(request):
    try:
        response = query_solr(q="id:"+request.POST['id'])
        return HttpResponse(json.dumps(response.docs[0]), content_type='application/json')
    except:
        raise Http404


def get_avaliables(request):
    try:
        data = json.dumps(request.POST)
        data = json.loads(data)
        response = {}
        library = Library.objects.filter(name=data['library'])[0]

        if data['field']:
            if library.id == 1:
                response = book_avaliable_uems(1.0, data['field'])
            elif library.id == 2:
                response = book_avaliable_ufgd(2.0, data['field'])
            return HttpResponse(response)
    except:
        raise Http404
    raise Http404
