# -*- coding: utf-8 -*-
from django.db import models
from django.contrib.gis.db import models
from django.contrib.gis.geos import Point
from location_field.models.spatial import LocationField


class Library(models.Model):
    """
        This model identifies the attributes of libraries registered in the system

        Attributes:
        * Name: Library Name
        * Location: Point for emblade in google maps
        * Amount of Books: Total number of registered books in this library
        * Regulation: ToDo
        * Objects: ToDo

    """

    class Meta:
        verbose_name = "Library"
        verbose_name_plural = "Libraries"

    name = models.CharField(max_length=200)
    amount_of_books = models.IntegerField(blank=True, null=True)
    regulation = models.URLField(blank=True, null=True)

    location = LocationField(based_fields=[name], zoom=7, default='Point(1.0 1.0)')
    objects = models.GeoManager()

    def __str__(self):
        return self.name


class ReportedError(models.Model):
    """
        Model to save error reported messages

        Error Mensage: ToDo

    """

    class Meta:
        verbose_name = "Reported Error"
        verbose_name_plural = "Reported Errors"

    error_message = models.TextField()

    def __str__(self):
        return self.error_message[0:45]
