import re
import pysolr
from django.conf import settings

from .models import Library


def search_Q(search={}):
    q = []
    qf = 'title^5 author^0.5' # priority to title and author

    q.append("(%s)" % (search['query']))

    if search['field']:
        qf = search['field']

    if search['library']:
        q.append("(library:%s)" % (str(Library.objects.get(name=search['library']).id)))

    if search['kind']:
        q.append(("(%s)") % (" OR ".join(["kind:" + lib for lib in search['kind']]))) # format kind search

    return " AND ".join(q), qf


def query_solr(q="*:*", qf='', wt='json', fl=['*'], start=0, rows=10, highlight=False):
    if len(q) > 0:
        solr = pysolr.Solr(settings.SOLR_BASEPATH + settings.SOLR_CORE)
        params = {
            'wt': wt,
            'fl': ', '.join(fl),
            'start': int(start),
            'rows': int(rows),
            'qf': qf,
            'lowercaseOperators': 'true',
            'stopwords': 'true',
            'defType': "edismax",
        }

        hl = {
            'hl': 'true',
            'hl.fl': "title",
            'hl.simple.pre': "<em>",
            'hl.simple.post': "</em>",
        }

        if highlight:
            params = dict(params.items() + hl.items())

        docs = solr.search(q=q, **params)
        for doc in docs.docs:
            doc['library'] = Library.objects.get(id=int(doc['library'])).name
        return docs
